<?php

/**
 * test case for experimental code
 *
 * @category  :category
 * @author    :author <*.*@sabel.jp>
 */
class Test_Experimental extends SabelTestCase
{  
  public static function suite()
  {
    return self::createSuite("Test_Experimental");
  }
  
  public function setUp()
  {
  }
  
  public function testExperimental()
  {
  }
}
