body {
  padding: 0;
  margin: 0;
  font: .8em Arial, sans-serif;
  line-height: 1.5em;
  background: #fff url("images/top.jpg") repeat-x top;
  color: #454545;
}

img {
  border: 0;
}

hr {
  height: 1px;
  border-style: none;
  color: #d0d0d0;
  background-color: #C0C0C0;
  margin: 10px 0;
}

/* header */
#header {
  padding-top: 20px;
  background: transparent;
  height: 101px;
  margin-left: 50px;
}

#header h1 {
  font: bold 1.8em Arial, Sans-Serif;
  padding: 4px 0;
  margin: 0;
  letter-spacing: 1px;
  color: #FFF;
}

#header h2 {
  font: bold 1.2em Arial, Sans-Serif;
  letter-spacing: 0px;
  color: #FFF;
  margin: 0;
  padding: 0;
}

/* main area */

div.content {
  margin: 0px auto;
  width: 770px;
}

#contents {
  background: #FFF;
  margin-bottom: 10px;
  width: 770px;
}

#contents table.sbl_basicTable {
  border-collapse: collapse;
  width: 90%;
}

#contents table.sbl_basicTable th {
  background-color: #e8efff;
  border: 1px solid #ccc;
}

#contents table.sbl_basicTable td {
  border: 1px solid #ccc;
  padding: 0 5px;
}

fieldset.formField {
  border: none;
}

div.sbl_error {
  margin-bottom: 15px;
  padding: 10px 10px 10px 15px;
  background-color: #ffe8e8;
  border: 3px solid #cc5555;
}

div.sbl_error ul {
  margin: 0 0 0 10px;
}

div.sbl_error ul li {
  padding-left: 5px;
  line-height: 1.4;
}

/* footer */
#footer {
  clear: both;
  border-top: 1px solid #DCDCDC;
  margin: 0 0 3em 0;
  color: #777;
  background: #fff;
}

#footer .right {
  float: right;
  margin-top: 10px;
  text-align: right;
  background: #FFF;
}
