<?php

/**
 * Sabel_Exception_ClassNotFound
 *
 * @category   Exception
 * @package    org.sabel.exception
 * @author     Ebine Yutaka <ebine.yutaka@sabel.jp>
 * @copyright  2004-2008 Mori Reo <mori.reo@sabel.jp>
 * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
 */
class Sabel_Exception_ClassNotFound extends Sabel_Exception_Runtime
{
  
}
