#!/bin/sh

sakle Migration development schema_test head
sakle Migration development student head
sakle Migration development student_course head
sakle Migration development course head
sakle Migration development session head
sakle Migration development sbl_storage head
sakle Migration development grandparents head
sakle Migration development parents head
sakle Migration development children head
sakle Migration development grandchildren head
