body {
  padding: 0;
  margin: 0;
  font: .74em Arial, sans-serif;
  line-height: 1.5em;
  background: #fff url("images/top.jpg") repeat-x top;
  color: #454545;
}

a {
  color: #DA3B3B;
  background: inherit;
}

a:hover {
  color: #DA3B3B;
  background: inherit;
  text-decoration: underline;
}

p {
  margin: 5px 0;
}

h1 {
  font: bold 1.8em Arial, Sans-Serif;
  padding: 8px 0 4px 0;
  margin: 0;
  letter-spacing: -1px;
}

h2 {
  font: bold 1.6em Arial, Sans-Serif;
  letter-spacing: -1px;
}

h3 {
  padding: 4px 0;
  margin: 0;
}

ul {
  margin: 0;
  padding : 0;
  list-style: none;
}

img {
  border: 0;
}

hr {
  height: 1px;
  border-style: none;
  color: #d0d0d0;
  background-color: #C0C0C0;
  margin: 10px 0;
}

.content {
  margin: 0px auto;
  width: 800px;
}

/* Top part */
#top {
  padding-top: 20px;
  background: transparent;
  height: 101px;
}

#top h1 {
  font: bold 1.8em Arial, Sans-Serif;
  padding: 8px 0 4px 0;
  margin: 0;
  letter-spacing: 1px;
  color: #FFF;
}

#top h2 {
  font: bold 1.2em Arial, Sans-Serif;
  letter-spacing: 0px;
  color: #FFF;
  margin: 0;
  padding: 0;
}

#top #icons {
  float: right;
  margin: 20px 0;
  padding: 0;
}

#top #icons img {
  padding-right: 2px;
  border: 0;
}

/* Second part */
#prec {
  height: 30px;
  margin: 0;
  padding: 0;
  background: #F6F6F6;
  border-bottom: 1px solid #DCDCDC;
  overflow: hidden;
  margin-bottom: 15px;
}

#wrap {
  margin: 0 auto;
  width: 800px;
}

#wrap #pic {
  float: right;
  background: #FF0000 url("images/mainimg.jpg") no-repeat;
  height: 199px;
  width: 589px;
  overflow: hidden;
}

#wrap #menu {
  padding: 0;
  margin: 0;
  background: inherit;
}

#wrap #menu a {
  padding-left: 25px;
  font: 0.9em Arial, Sans-Serif;
  text-decoration: none;
  background: #F6F6F6 url(images/lm-li.gif) no-repeat left;
}

#wrap #menu ul {
  padding: 10px 0;
}

#wrap #menu li {
  line-height: 26px;
  background: #F6F6F6 url("images/li-line.gif") no-repeat bottom left;
  list-style: none;
}

/*  Advertising */
#ad {
  padding-left: 95px;
  min-height: 10px;
  margin-bottom: 15px;
}

#ad a, #left_side a {
  background: #FFF;
  text-decoration: none;
}

#ad a:hover, #left_side a:hover {
  text-decoration: underline;
}

/* main area */
#main {

}

#contents {
  background: #FFF;
  margin-bottom: 10px;
  width: 800px;
}

#contents h3 {
  background: #FFF url(images/hbg.gif) repeat-x;
  height: 30px;
  padding: 5px 0 0 0px;
  margin: 0;
  font: bold 1.2em Arial, Sans-Serif;
}

#contents h3 span {
  background: url(images/h-art.gif) no-repeat left;
  padding: 1px 10px 0 23px;
  line-height: 22px;
  margin: 0;
}

#contents p {
  color: #454545;
  padding: 0 5px 0 5px;
  text-align: justify;
}

#contents img {
  float: left;
  padding: 0 10px 5px 0;
}

#contents blockquote {
  padding-left: 10px;
  border-left: 2px solid #DA4040;
  margin: 10px 0 10px 25px;
}

#contents .date {
  border-top: 1px dotted #ccc;
  padding: 5px 0;
  margin: 10px 0 25px 0;
  text-align: right;
}

#contents .rs {
  float: right;
  margin: 0 10px;
  border: 1px solid #ddd;
  padding: 5px;
  background: #f5f5f5;
}

#contents ul {
  list-style-position: inside;
  margin-left: 2px;
}

#contents ul li {
  list-style-type: square;
  margin-left: 15px;
}

#contents ul ul li {
  list-style: none;
  margin-left: 10px;
  list-style-type: lower-alpha;
  list-style-position: inside;
}

/* The footer */
#footer {
  clear: both;
  border-top: 1px solid #DCDCDC;
  margin: 0 0 3em 0;
  color: #777;
  background: #fff;
}

#footer .right {
  float: right;
  margin-top: 10px;
  text-align: right;
  background: #FFF;
}

#footer a {
  text-decoration: none;
  background: #FFF;
}

#config_area {
  display: none;
  position: absolute;
  top: 100px;
  width: 600px;
}

#black_layer {
  display: none;
  width: 100%;
  height: 100%;
  filter: alpha(opacity=65);
  background-color: #000;
  top: 0px;
  left: 0px;
  position: absolute;
  -moz-opacity: 0.65;
  opacity: 0.65;
}

#edit_textarea {
  height: 400px;
  width: 600px;
  font-family: 'Courier', 'MS Courier New',
               'Prestige', 'Everson Mono',
               'ＭＳ ゴシック', 'ＭＳ 明朝',
               'Osaka-等幅', 'Monaco';
}
