<?php

class Config_Addon implements Sabel_Config
{
  public function configure()
  {
    $addons = array();
    
    //$addons[] = "acl";
    //$addons[] = "flow";
    //$addons[] = "form";
    //$addons[] = "i18n";
    //$addons[] = "renderer";
    //$addons[] = "querylog";
    
    return $addons;
  }
}
